@@ -105,7 +105,7 @@ static void activateLookaheadLimiterConst(LADSPA_Handle instance) {
 	peak = 0.0f;
 	peak_dist = 1;
 	atten = 0.0f;
-	last_delay = -1.0f;
+	last_delay = 0.001 * fs;
 	plugin_data->amp_buffer = amp_buffer;
 	plugin_data->atten = atten;
 	plugin_data->buffer = buffer;
@@ -120,7 +120,7 @@ static void activateLookaheadLimiterConst(LADSPA_Handle instance) {
 }
 
 static void cleanupLookaheadLimiterConst(LADSPA_Handle instance) {
-#line 147 "lookahead_limiter_const_1906.xml"
+#line 138 "lookahead_limiter_const_1906.xml"
 	LookaheadLimiterConst *plugin_data = (LookaheadLimiterConst *)instance;
 	free(plugin_data->buffer);
 	free(plugin_data->amp_buffer);
@@ -195,7 +195,7 @@ static LADSPA_Handle instantiateLookaheadLimiterConst(
 	peak = 0.0f;
 	peak_dist = 1;
 	atten = 0.0f;
-	last_delay = -1.0f;
+	last_delay = 0.001 * fs;
 
 	plugin_data->amp_buffer = amp_buffer;
 	plugin_data->atten = atten;
@@ -255,16 +255,10 @@ static void runLookaheadLimiterConst(LADSPA_Handle instance, unsigned long sampl
 	const float max = DB_CO(limit);
 	float sig, gain;
 	float delay = last_delay;
-	float delay_delta;
+	const float delay_delta = (delay_s * fs - last_delay) / (sample_count - 1);
+	const unsigned int full_delay = f_round(BUFFER_TIME * fs);
 	float a, b;
 
-	if (delay < 0.0f) {
-	  delay = delay_s * fs;
-	  delay_delta = 0.0f;
-	} else {
-	  delay_delta = (delay_s * fs - last_delay) / (sample_count - 1);
-	}
-
 	for (pos = 0; pos < sample_count; pos++) {
 	  delay += delay_delta;
 	  buffer[(buffer_pos * 2) & buffer_mask] = in_1[pos];
@@ -272,49 +266,47 @@ static void runLookaheadLimiterConst(LADSPA_Handle instance, unsigned long sampl
 
 	  a = fabs(buffer[((buffer_pos + f_round(delay)) * 2) & buffer_mask]);
 	  b = fabs(buffer[((buffer_pos + f_round(delay)) * 2 + 1) & buffer_mask]);
+
 	  sig = a > b ? a : b;
-/* XXX
-	  sig = fabs(in_1[pos]) > fabs(in_2[pos]) ? fabs(in_1[pos]) :
-	          fabs(in_2[pos]);
-*/
 
 	  if (sig > max) {
-	    const float rel = lin2db(sig) - limit;
-
-	    if (rel / delay > peak / (float)peak_dist) {
+	    sig = lin2db(sig) - limit;
+	    if (sig / delay > peak / (float)peak_dist) {
 	      peak_dist = delay;
-	      peak = rel;
+	      peak = sig;
 	    }
 	  }
+	  if (sig > 0.0f && sig / (float)delay > peak / (float)peak_dist) {
+	    peak_dist = delay;
+	    peak = sig;
+	  }
 
 	  /* Incremenatlly approach the correct attenuation for the next peak */
 	  atten -= (atten - peak) / (float)(peak_dist + 1);
 
 	  if (peak_dist-- == 0) {
-	          peak_dist = f_round(delay);
+	          peak_dist = delay;
 	          peak = 0.0f;
 	  }
 
 	  /* Cacluate the apropriate gain reduction and write it back into the
 	   * buffer */
-	  gain = amp_buffer[(buffer_pos - f_round(delay)) & (buffer_len - 1)];
-	  amp_buffer[(buffer_pos - f_round(delay)) & (buffer_len - 1)] =
-	                                   1.0f / db2lin(atten);
+	  gain = 1.0f / db2lin(atten);
+	  amp_buffer[(buffer_pos - f_round(delay)) & (buffer_len - 1)] = gain;
 
-gain=1.0f / db2lin(atten);
+	  buffer_write(out_1[pos], buffer[(buffer_pos * 2 - 2) & buffer_mask] *
+	          gain);
+	  buffer_write(out_2[pos], buffer[(buffer_pos * 2 - 1) & buffer_mask] *
+	          gain);
 
-	  buffer_write(out_1[pos], buffer[(2 * (buffer_pos + 1)) &
-	                                  buffer_mask] * gain);
-	  buffer_write(out_2[pos], buffer[(2 * (buffer_pos + 1)+1) &
-	                                  buffer_mask] * gain);
+	  /* Ensure that the signal really can't be over the limit,
+	   * changes in the lookahead time can cause us to miss peaks */
 
-	  /* Ensure that the signal really can't be over the limit */
-
-#if 0
-XXX FIXME XXX
 	  if (out_1[pos] < -max) {
 	    buffer_write(out_1[pos], -max);
+//printf("c\n");
 	  } else if (out_1[pos] > max) {
+//printf("c\n");
 	    buffer_write(out_1[pos], max);
 	  }
 	  if (out_2[pos] < -max) {
@@ -322,7 +314,6 @@ XXX FIXME XXX
 	  } else if (out_2[pos] > max) {
 	    buffer_write(out_2[pos], max);
 	  }
-#endif
 
 	  buffer_pos++;
 	}
@@ -334,7 +325,7 @@ XXX FIXME XXX
 	plugin_data->last_delay = delay;
 
 	*(plugin_data->attenuation) = atten;
-	*(plugin_data->latency) = buffer_len - 1;
+	*(plugin_data->latency) = buffer_len;
 }
 #undef buffer_write
 #undef RUN_ADDING
@@ -385,16 +376,10 @@ static void runAddingLookaheadLimiterConst(LADSPA_Handle instance, unsigned long
 	const float max = DB_CO(limit);
 	float sig, gain;
 	float delay = last_delay;
-	float delay_delta;
+	const float delay_delta = (delay_s * fs - last_delay) / (sample_count - 1);
+	const unsigned int full_delay = f_round(BUFFER_TIME * fs);
 	float a, b;
 
-	if (delay < 0.0f) {
-	  delay = delay_s * fs;
-	  delay_delta = 0.0f;
-	} else {
-	  delay_delta = (delay_s * fs - last_delay) / (sample_count - 1);
-	}
-
 	for (pos = 0; pos < sample_count; pos++) {
 	  delay += delay_delta;
 	  buffer[(buffer_pos * 2) & buffer_mask] = in_1[pos];
@@ -402,49 +387,47 @@ static void runAddingLookaheadLimiterConst(LADSPA_Handle instance, unsigned long
 
 	  a = fabs(buffer[((buffer_pos + f_round(delay)) * 2) & buffer_mask]);
 	  b = fabs(buffer[((buffer_pos + f_round(delay)) * 2 + 1) & buffer_mask]);
+
 	  sig = a > b ? a : b;
-/* XXX
-	  sig = fabs(in_1[pos]) > fabs(in_2[pos]) ? fabs(in_1[pos]) :
-	          fabs(in_2[pos]);
-*/
 
 	  if (sig > max) {
-	    const float rel = lin2db(sig) - limit;
-
-	    if (rel / delay > peak / (float)peak_dist) {
+	    sig = lin2db(sig) - limit;
+	    if (sig / delay > peak / (float)peak_dist) {
 	      peak_dist = delay;
-	      peak = rel;
+	      peak = sig;
 	    }
 	  }
+	  if (sig > 0.0f && sig / (float)delay > peak / (float)peak_dist) {
+	    peak_dist = delay;
+	    peak = sig;
+	  }
 
 	  /* Incremenatlly approach the correct attenuation for the next peak */
 	  atten -= (atten - peak) / (float)(peak_dist + 1);
 
 	  if (peak_dist-- == 0) {
-	          peak_dist = f_round(delay);
+	          peak_dist = delay;
 	          peak = 0.0f;
 	  }
 
 	  /* Cacluate the apropriate gain reduction and write it back into the
 	   * buffer */
-	  gain = amp_buffer[(buffer_pos - f_round(delay)) & (buffer_len - 1)];
-	  amp_buffer[(buffer_pos - f_round(delay)) & (buffer_len - 1)] =
-	                                   1.0f / db2lin(atten);
+	  gain = 1.0f / db2lin(atten);
+	  amp_buffer[(buffer_pos - f_round(delay)) & (buffer_len - 1)] = gain;
 
-gain=1.0f / db2lin(atten);
+	  buffer_write(out_1[pos], buffer[(buffer_pos * 2 - 2) & buffer_mask] *
+	          gain);
+	  buffer_write(out_2[pos], buffer[(buffer_pos * 2 - 1) & buffer_mask] *
+	          gain);
 
-	  buffer_write(out_1[pos], buffer[(2 * (buffer_pos + 1)) &
-	                                  buffer_mask] * gain);
-	  buffer_write(out_2[pos], buffer[(2 * (buffer_pos + 1)+1) &
-	                                  buffer_mask] * gain);
+	  /* Ensure that the signal really can't be over the limit,
+	   * changes in the lookahead time can cause us to miss peaks */
 
-	  /* Ensure that the signal really can't be over the limit */
-
-#if 0
-XXX FIXME XXX
 	  if (out_1[pos] < -max) {
 	    buffer_write(out_1[pos], -max);
+//printf("c\n");
 	  } else if (out_1[pos] > max) {
+//printf("c\n");
 	    buffer_write(out_1[pos], max);
 	  }
 	  if (out_2[pos] < -max) {
@@ -452,7 +435,6 @@ XXX FIXME XXX
 	  } else if (out_2[pos] > max) {
 	    buffer_write(out_2[pos], max);
 	  }
-#endif
 
 	  buffer_pos++;
 	}
@@ -464,7 +446,7 @@ XXX FIXME XXX
 	plugin_data->last_delay = delay;
 
 	*(plugin_data->attenuation) = atten;
-	*(plugin_data->latency) = buffer_len - 1;
+	*(plugin_data->latency) = buffer_len;
 }
 
 void _init() {
