@@ -15,6 +15,7 @@ Copyright: GPL
 Group: Applications/Multimedia
 Source: http://plugin.org.uk/releases/$version/$package-$version.tar.gz
 BuildRoot: /var/tmp/%{name}-buildroot
+Prefix: /usr/local
 
 %description
 A set of audio plugins for LADSPA (see http://plugin.org.uk/ for more
@@ -41,7 +42,8 @@ rm -rf \$RPM_BUILD_ROOT
 
 EOB
 for $file (@files) {
-        print OUT "/usr/local/lib/ladspa/$file\n";
+        print OUT "%{prefix}/lib/ladspa/$file\n";
 }
+print OUT "%{prefix}/share/ladspa/rdf/swh-plugins.rdf";
 
 system("cp $package-$version.tar.gz /usr/src/redhat/SOURCES/");
