@@ -1,96 +1,33 @@
 #include <stdlib.h>
 #include <string.h>
+#ifndef WIN32
+#include "config.h"
+#endif
+
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#endif
+
+#define         _ISOC9X_SOURCE  1
+#define         _ISOC99_SOURCE  1
+#define         __USE_ISOC99    1
+#define         __USE_ISOC9X    1
+
 #include <math.h>
 
 #include "ladspa.h"
 
-#define LN_2_2 0.34657359 // ln(2)/2
-#define LIMIT(v,l,u) (v<l?l:(v>u?u:v))
+#ifdef WIN32
+#define _WINDOWS_DLL_EXPORT_ __declspec(dllexport)
+int bIsFirstTime = 1; 
+void _init(); // forward declaration
+#else
+#define _WINDOWS_DLL_EXPORT_ 
+#endif
 
-/* Biquad filter (adapted from code by Eli Brandt,
-   http://www.cs.cmu.edu/~eli/) */
+#line 10 "triple_para_1204.xml"
 
-typedef struct {
-        float a1;
-        float a2;
-        float b0;
-        float b1;
-        float b2;
-        float z0;
-        float z1;
-        float z2;
-} biquad;
-
-inline void biquad_init(biquad *f) {
-        f->z0 = 0.0f;
-        f->z1 = 0.0f;
-        f->z2 = 0.0f;
-}
-
-inline void eq_set_params(biquad *f, float fc, float gain, float bw,
-                          float fs) {
-        float w = 2.0f * M_PI * LIMIT(fc, 1.0, fs/2.0) / fs;
-        float cw = cos(w);
-        float sw = sin(w);
-        float J = pow(10.0f, gain / 40.0f);
-        float g = sw * sinh(LN_2_2 * LIMIT(bw, 0.0001, 4) * w / sw);
-        float a0r = 1.0f / (1.0 + (g / J));
-
-        f->b0 = (1.0 + (g * J)) * a0r;
-        f->b1 = (-2.0 * cw) * a0r;
-        f->b2 = (1.0 - (g * J)) * a0r;
-        f->a1 = -(f->b1);
-        f->a2 = ((g / J) - 1.0) * a0r;
-}
-
-inline void ls_set_params(biquad *f, float fc, float gain, float slope,
-                          float fs) {
-        float w = 2.0f * M_PI * LIMIT(fc, 1.0, fs/2.0) / fs;
-        float cw = cos(w);
-        float sw = sin(w);
-        float A = pow(10.0f, gain / 40.0f);
-        float b = sqrt(((1 + A * A) / LIMIT(slope, 0.0001, 1)) - ((A - 1.0) * (A - 1.0)));
-        float apc = cw * (A + 1.0);
-        float amc = cw * (A - 1.0);
-        float bs = b * sw;
-        float a0r = 1.0 / (A + 1.0 + amc + bs);
-
-        f->b0 = a0r * A * (A + 1.0 - amc + bs);
-        f->b1 = a0r * 2.0 * A * (A - 1.0 - apc);
-        f->b2 = a0r * A * (A + 1.0 - amc - bs);
-        f->a1 = a0r * 2.0 * (A - 1.0 + apc);
-        f->a2 = a0r * (-A - 1.0 - amc + bs);
-}
-
-inline void hs_set_params(biquad *f, float fc, float gain, float slope,
-                          float fs) {
-        float w = 2.0f * M_PI * LIMIT(fc, 1.0, fs/2.0) / fs;
-        float cw = cos(w);
-        float sw = sin(w);
-        float A = pow(10.0f, gain / 40.0f);
-        float b = sqrt(((1 + A * A) / LIMIT(slope, 0.0001, 1)) - ((A - 1.0) * (A - 1.0)));
-        float apc = cw * (A + 1.0);
-        float amc = cw * (A - 1.0);
-        float bs = b * sw;
-        float a0r = 1.0 / (A + 1.0 - amc + bs);
-
-        f->b0 = a0r * A * (A + 1.0 + amc + bs);
-        f->b1 = a0r * -2.0 * A * (A - 1.0 + apc);
-        f->b2 = a0r * (A + 1.0 + amc - bs);
-        f->a1 = a0r * -2.0 * (A - 1.0 - apc);
-        f->a2 = a0r * (-A - 1.0 + amc + bs);
-}
-
-inline float biquad_run(biquad *f, float in) {
-        float out;
-
-        f->z0 = in + (f->a1)*(f->z1) + (f->a2)*(f->z2);
-        out = (f->z0)*(f->b0) + (f->z1)*(f->b1) + (f->z2)*(f->b2);
-        f->z2 = f->z1;
-        f->z1 = f->z0;
-
-        return out;
-}
+#include "util/biquad.h"
 
 #define TRIPLEPARA_GAIN_L              0
 #define TRIPLEPARA_FC_L                1
@@ -110,7 +47,7 @@ inline float biquad_run(biquad *f, float in) {
 #define TRIPLEPARA_INPUT               15
 #define TRIPLEPARA_OUTPUT              16
 
-LADSPA_Descriptor *tripleParaDescriptor = NULL;
+static LADSPA_Descriptor *tripleParaDescriptor = NULL;
 
 typedef struct {
 	LADSPA_Data *gain_L;
@@ -135,7 +72,15 @@ typedef struct {
 	LADSPA_Data run_adding_gain;
 } TriplePara;
 
+_WINDOWS_DLL_EXPORT_
 const LADSPA_Descriptor *ladspa_descriptor(unsigned long index) {
+
+#ifdef WIN32
+	if (bIsFirstTime) {
+		_init();
+		bIsFirstTime = 0;
+	}
+#endif
 	switch (index) {
 	case 0:
 		return tripleParaDescriptor;
@@ -144,11 +89,11 @@ const LADSPA_Descriptor *ladspa_descriptor(unsigned long index) {
 	}
 }
 
-void activateTriplePara(LADSPA_Handle instance) {
+static void activateTriplePara(LADSPA_Handle instance) {
 	TriplePara *plugin_data = (TriplePara *)instance;
 	biquad *filters = plugin_data->filters;
 	float fs = plugin_data->fs;
-	fs = 0.0f;
+#line 32 "triple_para_1204.xml"
 	biquad_init(&filters[0]);
 	biquad_init(&filters[1]);
 	biquad_init(&filters[2]);
@@ -159,13 +104,14 @@ void activateTriplePara(LADSPA_Handle instance) {
 
 }
 
-void cleanupTriplePara(LADSPA_Handle instance) {
+static void cleanupTriplePara(LADSPA_Handle instance) {
+#line 68 "triple_para_1204.xml"
 	TriplePara *plugin_data = (TriplePara *)instance;
 	free(plugin_data->filters);
 	free(instance);
 }
 
-void connectPortTriplePara(
+static void connectPortTriplePara(
  LADSPA_Handle instance,
  unsigned long port,
  LADSPA_Data *data) {
@@ -227,13 +173,14 @@ void connectPortTriplePara(
 	}
 }
 
-LADSPA_Handle instantiateTriplePara(
+static LADSPA_Handle instantiateTriplePara(
  const LADSPA_Descriptor *descriptor,
  unsigned long s_rate) {
 	TriplePara *plugin_data = (TriplePara *)malloc(sizeof(TriplePara));
 	biquad *filters = NULL;
 	float fs;
 
+#line 21 "triple_para_1204.xml"
 	fs = s_rate;
 	
 	filters = calloc(5, sizeof(biquad));
@@ -257,79 +204,79 @@ LADSPA_Handle instantiateTriplePara(
 #define RUN_ADDING    0
 #define RUN_REPLACING 1
 
-void runTriplePara(LADSPA_Handle instance, unsigned long sample_count) {
+static void runTriplePara(LADSPA_Handle instance, unsigned long sample_count) {
 	TriplePara *plugin_data = (TriplePara *)instance;
 
 	/* Low-shelving gain (dB) (float value) */
-	LADSPA_Data gain_L = *(plugin_data->gain_L);
+	const LADSPA_Data gain_L = *(plugin_data->gain_L);
 
 	/* Low-shelving frequency (Hz) (float value) */
-	LADSPA_Data fc_L = *(plugin_data->fc_L);
+	const LADSPA_Data fc_L = *(plugin_data->fc_L);
 
 	/* Low-shelving slope (float value) */
-	LADSPA_Data bw_L = *(plugin_data->bw_L);
+	const LADSPA_Data bw_L = *(plugin_data->bw_L);
 
 	/* Band 1 gain (dB) (float value) */
-	LADSPA_Data gain_1 = *(plugin_data->gain_1);
+	const LADSPA_Data gain_1 = *(plugin_data->gain_1);
 
 	/* Band 1 frequency (Hz) (float value) */
-	LADSPA_Data fc_1 = *(plugin_data->fc_1);
+	const LADSPA_Data fc_1 = *(plugin_data->fc_1);
 
 	/* Band 1 bandwidth (octaves) (float value) */
-	LADSPA_Data bw_1 = *(plugin_data->bw_1);
+	const LADSPA_Data bw_1 = *(plugin_data->bw_1);
 
 	/* Band 2 gain (dB) (float value) */
-	LADSPA_Data gain_2 = *(plugin_data->gain_2);
+	const LADSPA_Data gain_2 = *(plugin_data->gain_2);
 
 	/* Band 2 frequency (Hz) (float value) */
-	LADSPA_Data fc_2 = *(plugin_data->fc_2);
+	const LADSPA_Data fc_2 = *(plugin_data->fc_2);
 
 	/* Band 2 bandwidth (octaves) (float value) */
-	LADSPA_Data bw_2 = *(plugin_data->bw_2);
+	const LADSPA_Data bw_2 = *(plugin_data->bw_2);
 
 	/* Band 3 gain (dB) (float value) */
-	LADSPA_Data gain_3 = *(plugin_data->gain_3);
+	const LADSPA_Data gain_3 = *(plugin_data->gain_3);
 
 	/* Band 3 frequency (Hz) (float value) */
-	LADSPA_Data fc_3 = *(plugin_data->fc_3);
+	const LADSPA_Data fc_3 = *(plugin_data->fc_3);
 
 	/* Band 3 bandwidth (octaves) (float value) */
-	LADSPA_Data bw_3 = *(plugin_data->bw_3);
+	const LADSPA_Data bw_3 = *(plugin_data->bw_3);
 
 	/* High-shelving gain (dB) (float value) */
-	LADSPA_Data gain_H = *(plugin_data->gain_H);
+	const LADSPA_Data gain_H = *(plugin_data->gain_H);
 
 	/* High-shelving frequency (Hz) (float value) */
-	LADSPA_Data fc_H = *(plugin_data->fc_H);
+	const LADSPA_Data fc_H = *(plugin_data->fc_H);
 
 	/* High-shelving slope (float value) */
-	LADSPA_Data bw_H = *(plugin_data->bw_H);
+	const LADSPA_Data bw_H = *(plugin_data->bw_H);
 
 	/* Input (array of floats of length sample_count) */
-	LADSPA_Data *input = plugin_data->input;
+	const LADSPA_Data * const input = plugin_data->input;
 
 	/* Output (array of floats of length sample_count) */
-	LADSPA_Data *output = plugin_data->output;
+	LADSPA_Data * const output = plugin_data->output;
 	biquad * filters = plugin_data->filters;
 	float fs = plugin_data->fs;
 
+#line 40 "triple_para_1204.xml"
 	unsigned long pos;
 	float in;
 	
-	#define SETUP_EQ(t,n,f,g,b) t##_set_params(&filters[n],f,g,b,fs);
-	
-	SETUP_EQ(ls, 0, fc_L, gain_L, bw_L);
-	SETUP_EQ(eq, 1, fc_1, gain_1, bw_1);
-	SETUP_EQ(eq, 2, fc_2, gain_2, bw_2);
-	SETUP_EQ(eq, 3, fc_3, gain_3, bw_3);
-	SETUP_EQ(hs, 4, fc_H, gain_H, bw_H);
+	ls_set_params(&filters[0], fc_L, gain_L, bw_L, fs);
+	eq_set_params(&filters[1], fc_1, gain_1, bw_1, fs);
+	eq_set_params(&filters[2], fc_2, gain_2, bw_2, fs);
+	eq_set_params(&filters[3], fc_3, gain_3, bw_3, fs);
+	hs_set_params(&filters[4], fc_H, gain_H, bw_H, fs);
 	
 	for (pos = 0; pos < sample_count; pos++) {
 	        in = biquad_run(&filters[0], input[pos]);
 	        in = biquad_run(&filters[1], in);
 	        in = biquad_run(&filters[2], in);
 	        in = biquad_run(&filters[3], in);
-	        buffer_write(output[pos], biquad_run(&filters[4], in));
+	        in = biquad_run(&filters[4], in);
+	        buffer_write(output[pos], in);
 	}
 }
 #undef buffer_write
@@ -340,84 +287,84 @@ void runTriplePara(LADSPA_Handle instance, unsigned long sample_count) {
 #define RUN_ADDING    1
 #define RUN_REPLACING 0
 
-void setRunAddingGainTriplePara(LADSPA_Handle instance, LADSPA_Data gain) {
+static void setRunAddingGainTriplePara(LADSPA_Handle instance, LADSPA_Data gain) {
 	((TriplePara *)instance)->run_adding_gain = gain;
 }
 
-void runAddingTriplePara(LADSPA_Handle instance, unsigned long sample_count) {
+static void runAddingTriplePara(LADSPA_Handle instance, unsigned long sample_count) {
 	TriplePara *plugin_data = (TriplePara *)instance;
 	LADSPA_Data run_adding_gain = plugin_data->run_adding_gain;
 
 	/* Low-shelving gain (dB) (float value) */
-	LADSPA_Data gain_L = *(plugin_data->gain_L);
+	const LADSPA_Data gain_L = *(plugin_data->gain_L);
 
 	/* Low-shelving frequency (Hz) (float value) */
-	LADSPA_Data fc_L = *(plugin_data->fc_L);
+	const LADSPA_Data fc_L = *(plugin_data->fc_L);
 
 	/* Low-shelving slope (float value) */
-	LADSPA_Data bw_L = *(plugin_data->bw_L);
+	const LADSPA_Data bw_L = *(plugin_data->bw_L);
 
 	/* Band 1 gain (dB) (float value) */
-	LADSPA_Data gain_1 = *(plugin_data->gain_1);
+	const LADSPA_Data gain_1 = *(plugin_data->gain_1);
 
 	/* Band 1 frequency (Hz) (float value) */
-	LADSPA_Data fc_1 = *(plugin_data->fc_1);
+	const LADSPA_Data fc_1 = *(plugin_data->fc_1);
 
 	/* Band 1 bandwidth (octaves) (float value) */
-	LADSPA_Data bw_1 = *(plugin_data->bw_1);
+	const LADSPA_Data bw_1 = *(plugin_data->bw_1);
 
 	/* Band 2 gain (dB) (float value) */
-	LADSPA_Data gain_2 = *(plugin_data->gain_2);
+	const LADSPA_Data gain_2 = *(plugin_data->gain_2);
 
 	/* Band 2 frequency (Hz) (float value) */
-	LADSPA_Data fc_2 = *(plugin_data->fc_2);
+	const LADSPA_Data fc_2 = *(plugin_data->fc_2);
 
 	/* Band 2 bandwidth (octaves) (float value) */
-	LADSPA_Data bw_2 = *(plugin_data->bw_2);
+	const LADSPA_Data bw_2 = *(plugin_data->bw_2);
 
 	/* Band 3 gain (dB) (float value) */
-	LADSPA_Data gain_3 = *(plugin_data->gain_3);
+	const LADSPA_Data gain_3 = *(plugin_data->gain_3);
 
 	/* Band 3 frequency (Hz) (float value) */
-	LADSPA_Data fc_3 = *(plugin_data->fc_3);
+	const LADSPA_Data fc_3 = *(plugin_data->fc_3);
 
 	/* Band 3 bandwidth (octaves) (float value) */
-	LADSPA_Data bw_3 = *(plugin_data->bw_3);
+	const LADSPA_Data bw_3 = *(plugin_data->bw_3);
 
 	/* High-shelving gain (dB) (float value) */
-	LADSPA_Data gain_H = *(plugin_data->gain_H);
+	const LADSPA_Data gain_H = *(plugin_data->gain_H);
 
 	/* High-shelving frequency (Hz) (float value) */
-	LADSPA_Data fc_H = *(plugin_data->fc_H);
+	const LADSPA_Data fc_H = *(plugin_data->fc_H);
 
 	/* High-shelving slope (float value) */
-	LADSPA_Data bw_H = *(plugin_data->bw_H);
+	const LADSPA_Data bw_H = *(plugin_data->bw_H);
 
 	/* Input (array of floats of length sample_count) */
-	LADSPA_Data *input = plugin_data->input;
+	const LADSPA_Data * const input = plugin_data->input;
 
 	/* Output (array of floats of length sample_count) */
-	LADSPA_Data *output = plugin_data->output;
+	LADSPA_Data * const output = plugin_data->output;
 	biquad * filters = plugin_data->filters;
 	float fs = plugin_data->fs;
 
+#line 40 "triple_para_1204.xml"
 	unsigned long pos;
 	float in;
 	
-	#define SETUP_EQ(t,n,f,g,b) t##_set_params(&filters[n],f,g,b,fs);
-	
-	SETUP_EQ(ls, 0, fc_L, gain_L, bw_L);
-	SETUP_EQ(eq, 1, fc_1, gain_1, bw_1);
-	SETUP_EQ(eq, 2, fc_2, gain_2, bw_2);
-	SETUP_EQ(eq, 3, fc_3, gain_3, bw_3);
-	SETUP_EQ(hs, 4, fc_H, gain_H, bw_H);
+	ls_set_params(&filters[0], fc_L, gain_L, bw_L, fs);
+	eq_set_params(&filters[1], fc_1, gain_1, bw_1, fs);
+	eq_set_params(&filters[2], fc_2, gain_2, bw_2, fs);
+	eq_set_params(&filters[3], fc_3, gain_3, bw_3, fs);
+	hs_set_params(&filters[4], fc_H, gain_H, bw_H, fs);
 	
 	for (pos = 0; pos < sample_count; pos++) {
 	        in = biquad_run(&filters[0], input[pos]);
 	        in = biquad_run(&filters[1], in);
 	        in = biquad_run(&filters[2], in);
 	        in = biquad_run(&filters[3], in);
-	        buffer_write(output[pos], biquad_run(&filters[4], in));
+	        in = biquad_run(&filters[4], in);
+	        buffer_write(output[pos], in);
 	}
 }
 
@@ -426,20 +373,29 @@ void _init() {
 	LADSPA_PortDescriptor *port_descriptors;
 	LADSPA_PortRangeHint *port_range_hints;
 
+#ifdef ENABLE_NLS
+#define D_(s) dgettext(PACKAGE, s)
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
+#else
+#define D_(s) (s)
+#endif
+
+
 	tripleParaDescriptor =
 	 (LADSPA_Descriptor *)malloc(sizeof(LADSPA_Descriptor));
 
 	if (tripleParaDescriptor) {
 		tripleParaDescriptor->UniqueID = 1204;
-		tripleParaDescriptor->Label = strdup("triplePara");
+		tripleParaDescriptor->Label = "triplePara";
 		tripleParaDescriptor->Properties =
 		 LADSPA_PROPERTY_HARD_RT_CAPABLE;
 		tripleParaDescriptor->Name =
-		 strdup("Triple band parametric with shelves");
+		 D_("Triple band parametric with shelves");
 		tripleParaDescriptor->Maker =
-		 strdup("Steve Harris <steve@plugin.org.uk>");
+		 "Steve Harris <steve@plugin.org.uk>";
 		tripleParaDescriptor->Copyright =
-		 strdup("GPL");
+		 "GPL";
 		tripleParaDescriptor->PortCount = 17;
 
 		port_descriptors = (LADSPA_PortDescriptor *)calloc(17,
@@ -460,9 +416,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_GAIN_L] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_GAIN_L] =
-		 strdup("Low-shelving gain (dB)");
+		 D_("Low-shelving gain (dB)");
 		port_range_hints[TRIPLEPARA_GAIN_L].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_0;
 		port_range_hints[TRIPLEPARA_GAIN_L].LowerBound = -70;
 		port_range_hints[TRIPLEPARA_GAIN_L].UpperBound = +30;
 
@@ -470,9 +426,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_FC_L] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_FC_L] =
-		 strdup("Low-shelving frequency (Hz)");
+		 D_("Low-shelving frequency (Hz)");
 		port_range_hints[TRIPLEPARA_FC_L].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_SAMPLE_RATE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_SAMPLE_RATE | LADSPA_HINT_DEFAULT_MINIMUM;
 		port_range_hints[TRIPLEPARA_FC_L].LowerBound = 0;
 		port_range_hints[TRIPLEPARA_FC_L].UpperBound = 0.5;
 
@@ -480,9 +436,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_BW_L] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_BW_L] =
-		 strdup("Low-shelving slope");
+		 D_("Low-shelving slope");
 		port_range_hints[TRIPLEPARA_BW_L].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_MIDDLE;
 		port_range_hints[TRIPLEPARA_BW_L].LowerBound = 0;
 		port_range_hints[TRIPLEPARA_BW_L].UpperBound = 1;
 
@@ -490,9 +446,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_GAIN_1] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_GAIN_1] =
-		 strdup("Band 1 gain (dB)");
+		 D_("Band 1 gain (dB)");
 		port_range_hints[TRIPLEPARA_GAIN_1].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_0;
 		port_range_hints[TRIPLEPARA_GAIN_1].LowerBound = -70;
 		port_range_hints[TRIPLEPARA_GAIN_1].UpperBound = +30;
 
@@ -500,9 +456,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_FC_1] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_FC_1] =
-		 strdup("Band 1 frequency (Hz)");
+		 D_("Band 1 frequency (Hz)");
 		port_range_hints[TRIPLEPARA_FC_1].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_SAMPLE_RATE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_SAMPLE_RATE | LADSPA_HINT_DEFAULT_LOW;
 		port_range_hints[TRIPLEPARA_FC_1].LowerBound = 0;
 		port_range_hints[TRIPLEPARA_FC_1].UpperBound = 0.5;
 
@@ -510,9 +466,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_BW_1] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_BW_1] =
-		 strdup("Band 1 bandwidth (octaves)");
+		 D_("Band 1 bandwidth (octaves)");
 		port_range_hints[TRIPLEPARA_BW_1].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_1;
 		port_range_hints[TRIPLEPARA_BW_1].LowerBound = 0;
 		port_range_hints[TRIPLEPARA_BW_1].UpperBound = 4;
 
@@ -520,9 +476,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_GAIN_2] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_GAIN_2] =
-		 strdup("Band 2 gain (dB)");
+		 D_("Band 2 gain (dB)");
 		port_range_hints[TRIPLEPARA_GAIN_2].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_0;
 		port_range_hints[TRIPLEPARA_GAIN_2].LowerBound = -70;
 		port_range_hints[TRIPLEPARA_GAIN_2].UpperBound = +30;
 
@@ -530,9 +486,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_FC_2] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_FC_2] =
-		 strdup("Band 2 frequency (Hz)");
+		 D_("Band 2 frequency (Hz)");
 		port_range_hints[TRIPLEPARA_FC_2].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_SAMPLE_RATE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_SAMPLE_RATE | LADSPA_HINT_DEFAULT_MIDDLE;
 		port_range_hints[TRIPLEPARA_FC_2].LowerBound = 0;
 		port_range_hints[TRIPLEPARA_FC_2].UpperBound = 0.5;
 
@@ -540,9 +496,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_BW_2] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_BW_2] =
-		 strdup("Band 2 bandwidth (octaves)");
+		 D_("Band 2 bandwidth (octaves)");
 		port_range_hints[TRIPLEPARA_BW_2].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_1;
 		port_range_hints[TRIPLEPARA_BW_2].LowerBound = 0;
 		port_range_hints[TRIPLEPARA_BW_2].UpperBound = 4;
 
@@ -550,9 +506,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_GAIN_3] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_GAIN_3] =
-		 strdup("Band 3 gain (dB)");
+		 D_("Band 3 gain (dB)");
 		port_range_hints[TRIPLEPARA_GAIN_3].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_0;
 		port_range_hints[TRIPLEPARA_GAIN_3].LowerBound = -70;
 		port_range_hints[TRIPLEPARA_GAIN_3].UpperBound = +30;
 
@@ -560,9 +516,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_FC_3] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_FC_3] =
-		 strdup("Band 3 frequency (Hz)");
+		 D_("Band 3 frequency (Hz)");
 		port_range_hints[TRIPLEPARA_FC_3].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_SAMPLE_RATE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_SAMPLE_RATE | LADSPA_HINT_DEFAULT_HIGH;
 		port_range_hints[TRIPLEPARA_FC_3].LowerBound = 0;
 		port_range_hints[TRIPLEPARA_FC_3].UpperBound = 0.5;
 
@@ -570,9 +526,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_BW_3] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_BW_3] =
-		 strdup("Band 3 bandwidth (octaves)");
+		 D_("Band 3 bandwidth (octaves)");
 		port_range_hints[TRIPLEPARA_BW_3].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_1;
 		port_range_hints[TRIPLEPARA_BW_3].LowerBound = 0;
 		port_range_hints[TRIPLEPARA_BW_3].UpperBound = 4;
 
@@ -580,9 +536,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_GAIN_H] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_GAIN_H] =
-		 strdup("High-shelving gain (dB)");
+		 D_("High-shelving gain (dB)");
 		port_range_hints[TRIPLEPARA_GAIN_H].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_0;
 		port_range_hints[TRIPLEPARA_GAIN_H].LowerBound = -70;
 		port_range_hints[TRIPLEPARA_GAIN_H].UpperBound = +30;
 
@@ -590,9 +546,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_FC_H] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_FC_H] =
-		 strdup("High-shelving frequency (Hz)");
+		 D_("High-shelving frequency (Hz)");
 		port_range_hints[TRIPLEPARA_FC_H].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_SAMPLE_RATE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_SAMPLE_RATE | LADSPA_HINT_DEFAULT_MAXIMUM;
 		port_range_hints[TRIPLEPARA_FC_H].LowerBound = 0;
 		port_range_hints[TRIPLEPARA_FC_H].UpperBound = 0.5;
 
@@ -600,9 +556,9 @@ void _init() {
 		port_descriptors[TRIPLEPARA_BW_H] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
 		port_names[TRIPLEPARA_BW_H] =
-		 strdup("High-shelving slope");
+		 D_("High-shelving slope");
 		port_range_hints[TRIPLEPARA_BW_H].HintDescriptor =
-		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
+		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_MIDDLE;
 		port_range_hints[TRIPLEPARA_BW_H].LowerBound = 0;
 		port_range_hints[TRIPLEPARA_BW_H].UpperBound = 1;
 
@@ -610,7 +566,7 @@ void _init() {
 		port_descriptors[TRIPLEPARA_INPUT] =
 		 LADSPA_PORT_INPUT | LADSPA_PORT_AUDIO;
 		port_names[TRIPLEPARA_INPUT] =
-		 strdup("Input");
+		 D_("Input");
 		port_range_hints[TRIPLEPARA_INPUT].HintDescriptor =
 		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
 		port_range_hints[TRIPLEPARA_INPUT].LowerBound = -1.0;
@@ -620,7 +576,7 @@ void _init() {
 		port_descriptors[TRIPLEPARA_OUTPUT] =
 		 LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO;
 		port_names[TRIPLEPARA_OUTPUT] =
-		 strdup("Output");
+		 D_("Output");
 		port_range_hints[TRIPLEPARA_OUTPUT].HintDescriptor =
 		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
 		port_range_hints[TRIPLEPARA_OUTPUT].LowerBound = -1.0;
@@ -638,16 +594,8 @@ void _init() {
 }
 
 void _fini() {
-	int i;
-
 	if (tripleParaDescriptor) {
-		free((char *)tripleParaDescriptor->Label);
-		free((char *)tripleParaDescriptor->Name);
-		free((char *)tripleParaDescriptor->Maker);
-		free((char *)tripleParaDescriptor->Copyright);
 		free((LADSPA_PortDescriptor *)tripleParaDescriptor->PortDescriptors);
-		for (i = 0; i < tripleParaDescriptor->PortCount; i++)
-			free((char *)(tripleParaDescriptor->PortNames[i]));
 		free((char **)tripleParaDescriptor->PortNames);
 		free((LADSPA_PortRangeHint *)tripleParaDescriptor->PortRangeHints);
 		free(tripleParaDescriptor);
